use std::error::Error;

fn factorial(num: usize) -> usize {
    let mut fact = 1;
    for n in 1..=num {
        fact *= n;
    }
    fact
}

fn binomial(n: usize, k: usize) -> usize {
    if k > n {
        0
    } else {
        factorial(n) / (factorial(k) * factorial(n - k))
    }
}

pub fn nth_lex<T: Clone + Ord>(
    mut elements: Vec<T>,
    k: usize,
    nth: usize,
) -> Result<Vec<T>, Box<dyn Error>> {
    elements.sort();
    let num_elements = elements.len();
    let num_combinations = binomial(num_elements, k);
    if nth > num_combinations || k > num_elements || nth == 0 || k == 0 {
        return Err(Box::from("Out of bounds"));
    }
    let mut i = 0;
    let mut remaining_k = k;
    let mut comb = Vec::new();
    let mut remainder = nth - 1;
    while remaining_k > 0 {
        // Count the number of combinations that start with elements[i]
        // example with n = 5, k = 2
        // nth <= 4 select first
        // nth <= 7 select second
        // nth <= 9 select third
        // nth == 10 select fourth
        let count = binomial(num_elements - i - 1, remaining_k - 1);
        if remainder < count {
            // If the nth combination is within the count, pick this element
            comb.push(elements[i].clone());
            remaining_k -= 1;
        } else {
            remainder -= count;
        }
        i += 1;
    }
    Ok(comb)
}

#[derive(Clone)]
pub struct Combinator<T: Clone + Ord> {
    pub current: Vec<T>,
    pub k: usize,
    idx: usize,
}

impl<T: Clone + Ord> Combinator<T> {
    pub fn new(elements: Vec<T>, k: usize) -> Result<Combinator<T>, Box<dyn Error>> {
        if k > elements.len() || k == 0 {
            return Err(Box::from("Out of bounds"));
        }
        Ok(Self {
            current: elements,
            k,
            idx: 0,
        })
    }
}

impl<T: Clone + Ord> Iterator for Combinator<T> {
    type Item = Vec<T>;

    fn next(&mut self) -> Option<Self::Item> {
        let num_elements = self.current.len();
        let num_combinations = binomial(num_elements, self.k);
        if self.idx == num_combinations {
            return None;
        }
        let mut i = 0;
        let mut remaining_k = self.k;
        let mut comb = Vec::new();
        let mut remainder = self.idx - 1;
        while remaining_k > 0 {
            // Count the number of combinations that start with elements[i]
            // example with n = 5, k = 2
            // nth <= 4 select first
            // nth <= 7 select second
            // nth <= 9 select third
            // nth == 10 select fourth
            let count = binomial(num_elements - i - 1, remaining_k - 1);
            if remainder < count {
                // If the nth combination is within the count, pick this element
                comb.push(self.current[i].clone());
                remaining_k -= 1;
            } else {
                remainder -= count;
            }
            i += 1;
        }
        self.idx += 1;
        self.current = comb;
        Some(self.current.clone())
    }
}
