use std::{collections::HashSet, error::Error};

use crate::number::Number;

#[derive(Debug, PartialEq, Eq)]
pub struct Fraction {
    pub numerator: Number,
    pub denominator: Number,
}

pub struct Rational {}

impl Fraction {
    pub fn new(numerator: Number, denominator: Number) -> Result<Self, Box<dyn Error>> {
        if denominator == 0.into() {
            return Err(Box::from("Division by 0"));
        }
        let mut f = Fraction {
            numerator,
            denominator,
        };
        f.reduce();
        Ok(f)
    }

    pub fn reduce(&mut self) {
        let gcd = Number::gcd(self.numerator.clone(), self.denominator.clone());
        if gcd == self.denominator.clone() {
            return;
        }
        self.numerator = self.numerator.clone() / gcd.clone();
        self.denominator = self.denominator.clone() / gcd.clone();
    }

    pub fn get_repeating(&self) -> Number {
        let mut memory = HashSet::new();
        let mut div_with = self.numerator.clone();
        let mut repetition = Vec::new();
        let num_len = self.numerator.digits.len();
        for _ in 0..num_len {
            div_with = div_with * 10;
        }
        loop {
            println!("{div_with}/{}", self.denominator.clone());
            let (div, rem) =
                Number::div_with_rem(div_with.clone(), self.denominator.clone()).unwrap();
            println!("div: {div}, rem: {rem}");
            if div == 0.into() || rem == 0.into() || memory.contains(&rem) {
                repetition.push(div.digits[0]);
                return Number {
                    digits: repetition,
                    sign: crate::number::Sign::Positif,
                };
            }
            memory.insert(rem.clone());
            repetition.push(div.digits[0]);
            div_with = rem * 10;
        }
    }
}

#[cfg(test)]
mod test {
    use crate::number::Number;

    use super::Fraction;

    #[test]
    fn test_reduce() {
        let f = Fraction::new(120.into(), 90.into()).unwrap();
        assert_eq!(
            f,
            Fraction {
                numerator: 4.into(),
                denominator: 3.into()
            }
        );
    }

    #[test]
    fn test_repeating() {
        let f = Fraction::new(1.into(), 7.into()).unwrap();
        let rep = f.get_repeating();
        assert_eq!(rep, Number::from("142857"));
    }
}
