use std::error::Error;

#[derive(Debug)]
pub struct Grid<T: Clone>(pub Vec<Vec<T>>);

impl<T: Clone> Grid<T> {
    pub fn new(grid: Vec<Vec<T>>) -> Result<Self, Box<dyn Error>> {
        let row_length = grid[0].len();
        for row in &grid {
            if row_length != row.len() {
                return Err(Box::from("Rows need to all be equal in length"));
            }
        }
        Ok(Grid(grid))
    }

    pub fn invert(&mut self) -> Self {
        let height = self.0.len();
        let width = self.0[0].len();

        let mut new_grid = Vec::with_capacity(width);

        for col_idx in 0..width {
            let mut new_row = Vec::with_capacity(height);
            for row_idx in 0..height {
                new_row.push(self.0[row_idx][col_idx].clone());
            }
            new_grid.push(new_row);
        }
        self.0 = new_grid;
        Grid(self.0.clone())
    }
}
